{% if ip_ver == 4 %}
{% set max_pref_length = 32 %}
{% else %}
{% set max_pref_length = 128 %}
{% endif %}

{% include "header.j2" %}


{% include "rpki.j2" %}


{% include "common.j2" %}


{% include "irrdb.j2" %}


{% include "clients.j2" %}
 

include "*.local";
{% if ip_ver == 4 %}
include "*.local4";
{% else %}
include "*.local6";
{% endif %}
